#//-----------------------------------------------
#//TUSB-S03CN4用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_07_11　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbs03cn4dll = cdll.LoadLibrary("TUSBSCN4.DLL")

#######  dll function definitions (do not call directry from user codes)
tusbs03cn4dll.Tusbs03cn4_Device_Open.restype = c_short
tusbs03cn4dll.Tusbs03cn4_Device_Open.argtypes = (c_short,)
tusbs03cn4dll.Tusbs03cn4_Device_Close.restype = None
tusbs03cn4dll.Tusbs03cn4_Device_Close.argtypes = (c_short,)
tusbs03cn4dll.Tusbs03cn4_Counter_Start.restype = c_short
tusbs03cn4dll.Tusbs03cn4_Counter_Start.argtypes = (c_short,c_ubyte,c_ubyte,c_ubyte,c_ubyte)
tusbs03cn4dll.Tusbs03cn4_Counter_Stop.restype = c_short
tusbs03cn4dll.Tusbs03cn4_Counter_Stop.argtypes = (c_short,c_ubyte)
tusbs03cn4dll.Tusbs03cn4_Fifo_Cnt.restype = c_short
tusbs03cn4dll.Tusbs03cn4_Fifo_Cnt.argtypes = (c_short,c_ubyte,POINTER(c_ushort))
tusbs03cn4dll.Tusbs03cn4_Fifo_Read.restype = c_short
tusbs03cn4dll.Tusbs03cn4_Fifo_Read.argtypes = (c_short,c_ubyte,POINTER(c_ushort),POINTER(c_uint),POINTER(c_uint))

#######  support functions (do not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

######## def functions   ###############################################

def Tusbs03cn4_Device_Open(DevId):
    """Open Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    
    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId)):
        return tusbs03cn4dll.Tusbs03cn4_Device_Open(DevId)
    else:
        return 1

def Tusbs03cn4_Device_Close(DevId):
    """Close Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    """
    if( IsValidId(DevId)):
        tusbs03cn4dll.Tusbs03cn4_Device_Close(DevId)

def Tusbs03cn4_Counter_Start(DevId,Ch,Edge,DivI,DivC):
    """Start Counter
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Target Channel 0~2
    Edge            ----    Count Edge Direction 0:Up 1:Down
    DivI            ----    Input Divider Select 0~5
    DivC            ----    Internal Timer Divider Select 0~4
    
    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Ch,0,2): return 8
    if not IsInRange(Edge,0,1): return 8
    if not IsInRange(DivI,0,5): return 8
    if not IsInRange(DivC,0,4): return 8
    return tusbs03cn4dll.Tusbs03cn4_Counter_Start(DevId,Ch,Edge,DivI,DivC)

def Tusbs03cn4_Counter_Stop(DevId,Ch):
    """Stop Counter
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Target Channel 0~2
    
    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Ch,0,2): return 8
    return tusbs03cn4dll.Tusbs03cn4_Counter_Stop(DevId,Ch)

def Tusbs03cn4_Fifo_Cnt(DevId,Ch):
    """Read FIFO Data count
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Target Channel 0~2

    Returns:
    int,            ----    Result code
    int,            ----    FIFO Count
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Ch,0,2): return 8
    DLen = c_ushort()
    ret = tusbs03cn4dll.Tusbs03cn4_Fifo_Cnt(DevId,Ch,byref(DLen))
    if ret != 0:return ret,None
    return 0,DLen.value

def Tusbs03cn4_Fifo_Read(DevId,Ch,ReqLen):
    """Read FIFO Count Datas
    
    Arguments:
    DevId           ----    Device Id 0~15
    Ch              ----    Target Channel 0~2
    ReqLen          ----    Request FIFO Data Length 1~512

    Returns:
    int,            ----    Result code
    [int,],         ----    FIFO Count Data List
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Ch,0,2): return 8
    if not IsInRange(ReqLen,1,512): return 8
    Len = c_ushort(ReqLen)
    DataH = ( c_uint * ReqLen)()
    DataL = ( c_uint * ReqLen)()
    p_DataH = cast(pointer(DataH),POINTER(c_uint))
    p_DataL = cast(pointer(DataL),POINTER(c_uint))
    ret = tusbs03cn4dll.Tusbs03cn4_Fifo_Read(DevId,Ch,byref(Len),p_DataH,p_DataL)
    if ret != 0: return ret,None
    FifoData = list()
    Len = Len.value
    for cnt in range(Len):
        FifoData.append((p_DataH[cnt] << 16 ) + p_DataL[cnt])
    return 0,FifoData

# Error Messages from return coddes
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバがインストールされていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            7:"オープンされていません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"

