#//-----------------------------------------------
#//TUSB-S03CN4用サンプルプログラム
#//Python 用モジュール
#//
#//2024_07_11　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import TUSBSCN4
from functools import partial
from SpFuncs import *
import sys

class Tusbs03cn4_Device:
    """TUSB-S03CN4 デバイス操作クラス フォーム"""

    DevId = None                    # デバイスID
    mon_t = None                    # カウントモニタータイマー
    PreCount = [0,0,0]              # 差分計算用　前回のカウント値
    LOG_BTN_START = "ログ開始"      # ログ開始停止ボタン開始表示文字列
    LOG_BTN_STOP = "ログ停止"       # ログ開始停止ボタン停止表示文字列

    def __init__(self,DevId):
        """イニシャライザ
        
        Arguments:
        DevId           ----    Device Id 0~15
        """
        self.formgen()
        ret = TUSBSCN4.Tusbs03cn4_Device_Open(DevId)
        if ret == 0:    # 正常にオープン
            self.DevId = DevId                                      # デバイスIDをクラスメンバ変数に保持する
            self.form.protocol("WM_DELETE_WINDOW",self.DevClose)
            self.EnableDisableControl(0,True)                       # 画面表示ウィジットの初期化
            self.EnableDisableControl(1,True)
            self.EnableDisableControl(2,True)
            self.mon_t = intTT(0.3,self.Monitor)                    # モニタータイマー作成、コールバック関数登録
        else:
            ShowInfo(TUSBSCN4.GetErrMessage(ret))                   # 失敗エラーメッセージ表示
            self.form.destroy()                                     # フォームを閉じる 

    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            if self.mon_t != None:
                self.mon_t.Stop()                                   # モニタタイマー終了
            TUSBSCN4.Tusbs03cn4_Device_Close(self.DevId)            # デバイスクローズ
            self.form.destroy()                                     # フォームを閉じる
            self.DevId = None

    def EnableDisableControl(self,Ch,State):
        """ログ開始、停止時でのウィジットの表示状態設定
        
        Arguments:
        Ch          ----    設定するチャンネル 0~2
        State       ----    動作状態 True:待機時 False:カウント中
        """
        if State:
            self.BtnStart[Ch]["text"] = self.LOG_BTN_START
            self.CmbbTimSel[Ch]['state'] = tk.NORMAL
            self.CmbbDivSel[Ch]['state'] = tk.NORMAL
            self.RbtnInEdge[Ch][0]['state'] = tk.NORMAL
            self.RbtnInEdge[Ch][1]['state'] = tk.NORMAL
        else:
            self.BtnStart[Ch]["text"] = self.LOG_BTN_STOP
            self.CmbbTimSel[Ch]['state'] = tk.DISABLED
            self.CmbbDivSel[Ch]['state'] = tk.DISABLED
            self.RbtnInEdge[Ch][0]['state'] = tk.DISABLED
            self.RbtnInEdge[Ch][1]['state'] = tk.DISABLED
    
    def LogStart(self,Ch):
        """ログを開始する"""
        self.LstbxCount[Ch].delete(0,tk.END)    # 過去のリストボックスログを消す
        EdgeSel = self.RbtnInEdgeV[Ch].get()
        DivI = self.CmbbDivSel[Ch].current()
        DivC = self.CmbbTimSel[Ch].current()
        ret = TUSBSCN4.Tusbs03cn4_Counter_Start(self.DevId,Ch,EdgeSel,DivI,DivC)    # ログ開始
        if ret != 0:
            ShowInfo(TUSBSCN4.GetErrMessage(ret))
            return
        self.PreCount[Ch] = 0                   # 差分カウント値算出のための前回値リセット
        self.mon_t.Start()                      # タイマースタート
        self.EnableDisableControl(Ch,False)
        self.LbldMessage[Ch]["text"] = "ログ中"
        LabelStatusCol(self.LbldFifoLen[Ch],False)
        
    def LogStop(self,Ch):
        """ログを停止する"""
        self.mon_t.Stop()                                       # タイマー停止
        ret = TUSBSCN4.Tusbs03cn4_Counter_Stop(self.DevId,Ch)   # カウンタ停止
        if ret != 0:
            ShowInfo(TUSBSCN4.GetErrMessage(ret))
            return
        self.EnableDisableControl(Ch,True)
        self.LbldMessage[Ch]["text"] = "ログ終了"  

    def CntCheck(self,Ch):
        """カウンタの値の読み込みと表示"""
        ret = TUSBSCN4.Tusbs03cn4_Fifo_Cnt(self.DevId,Ch)       # FIFOデータ数確認
        if ret[0] != 0:
            self.mon_t.Stop()   # タイマー停止
            ShowInfo(TUSBSCN4.GetErrMessage(ret[0]))
            return
        FifoCnt = ret[1]
        if FifoCnt >= 512:  # データバッファがいっぱいになっていたら(オーバーフロー)
            LabelStatusCol(self.LbldFifoLen[Ch],True)               # FIFOデータ数表示のバックグラウンドカラーを変えて
            self.LbldMessage[Ch]['text'] = "FIFOオーバーフロー"     # オーバーフローと表示する
        self.LbldFifoLen[Ch]['text'] = str(int(( FifoCnt / 512.0 ) * 100.0)) + '%'
        if FifoCnt != 0:                                                # 1つ以上のログデータがある
            ret = TUSBSCN4.Tusbs03cn4_Fifo_Read(self.DevId,Ch,FifoCnt)  # ログデータ読み込み
            if ret[0] != 0:
                self.mon_t.Stop()                       # タイマー停止
                ShowInfo(TUSBSCN4.GetErrMessage(ret[0]))
                return
            FifoData = ret[1]           # FIFOから取得したデータ
            FifoCnt = len(FifoData)     # 取得したデータ数
            for cnt in range(FifoCnt):
                # カウント差分計算
                if FifoData[cnt] < self.PreCount[Ch]:   # カウンタがオーバーフローした
                    tmpStr = str(4294967296 + FifoData[cnt] - self.PreCount[Ch])
                else:
                    tmpStr = str(FifoData[cnt] - self.PreCount[Ch])
                self.PreCount[Ch] = FifoData[cnt]   # 前回カウント値の記録
                # ログ表示リストボックスに値を追加
                ItemCountStr = "[" + str(self.LstbxCount[Ch].size()) + "]   "
                self.LstbxCount[Ch].insert(tk.END,ItemCountStr+tmpStr)
                if self.LstbxCount[Ch].size() >= 32767:
                    self.LogStop(Ch)    # ログ停止
                    self.LbldMessage[Ch]['text'] = "取込数制限で停止"
                    self.LstbxCount[Ch].yview(tk.END)
                    return
            self.LstbxCount[Ch].yview(tk.END)

    # timer -------------------------------------
    def Monitor(self):
        """モニタータイマー"""
        for ch in range(3):
            if self.BtnStart[ch]["text"] == "ログ停止":
                self.CntCheck(ch)

    # ボタン等ウィジットの操作イベントハンドラ ----------------------
    def BtnStart_on(self,Ch):
        """開始、停止ボタンハンドラ"""
        if self.BtnStart[Ch]["text"] == self.LOG_BTN_START: # カウント開始時
            self.LogStart(Ch)
        else:                                       # カウント停止時
            self.LogStop(Ch)

    # フォームとウィジェットを生成する処理---------------------------
    def formgen(self):
        self.form = tk.Tk()
        form = self.form
        form.geometry('649x453')
        form.title('TUSB-S03CN4 Sample Program')
        self.TimSelDict ={
            "100ns":0,
            "1us":1,
            "10us":2,
            "100us":3,
            "1ms":4,
        }
        self.DivSelDict ={
            "1/1":0,
            "1/10":1,
            "1/100":2,
            "1/1000":3,
            "1/10000":4,
            "1/100000":5,
        }
        self.InEdgeDict ={
            "立上り":0,
            "立下り":1,
        }
        self.CmbbTimSel = GenList(3,0,None)
        self.CmbbDivSel = GenList(3,0,None)
        self.RbtnInEdge = GenList(3,0,None)
        self.RbtnInEdgeV = GenTkVarList(3,0,tk.IntVar,0)
        self.LbldFifoLen = GenList(3,0,None)
        self.LbldMessage = GenList(3,0,None)
        self.BtnStart = GenList(3,0,None)
        self.LstbxCount = GenList(3,0,None)
        for ch in range(3):
            chframe = GetLFrame(form,"INPUT"+str(ch+1),0.95,0.3,0.025,0.02+ch*0.32)
            GetLabel(chframe,"基準タイマー",0.02,0.05)
            self.CmbbTimSel[ch] = GetComboBox(chframe,list(self.TimSelDict.keys()),0.15,0.02,0.25)
            GetLabel(chframe,"入力分周器",0.2,0.05)
            self.CmbbDivSel[ch] = GetComboBox(chframe,list(self.DivSelDict.keys()),0.15,0.2,0.25)
            ineframe = GetLFrame(chframe,"入力エッジ",0.15,0.5,0.02,0.4)
            self.RbtnInEdge[ch] = GetRadioButtons(ineframe,self.InEdgeDict,0.1,0.2,0.0,0.47,self.RbtnInEdgeV[ch])
            GetLabel(chframe,"FIFOバッファ使用率",0.18,0.55)
            self.LbldFifoLen[ch] = GetLabelDisp(chframe,"",0.15,0.35,0.55)
            self.LbldMessage[ch] = GetLabelDisp(chframe,"",0.32,0.18,0.8)
            self.BtnStart[ch] = GetButton(chframe,self.LOG_BTN_START,partial(self.BtnStart_on,Ch=ch),0.14,0.43,0.25)
            self.LstbxCount[ch]= GenListBox(chframe,0.45,0.8,0.52,0.1)
            self.LstbxCount[ch].config(justify="left")         # 文字を左寄せ

# 実行部 ---------------------------------
if __name__=="__main__":
    DevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            DevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusbs03cn4_Device(DevId=DevId)
    tmp.form.mainloop()

